#!/usr/bin/env python
# $Id$
'''
Class that displays percentage based progress bar.
Uses python generator function as input
'''
from GeneratorTask import GeneratorTask
import pygtk
pygtk.require('2.0')
import gtk

class ProgressBar:
    def set_progress_bar_fraction(self, fraction):
        self.pbar.set_fraction(fraction)
        if fraction < 0.1:
            spacing = ' ' * 4
        elif fraction == 1.0:
            spacing = ' ' * 2
            self.button.set_label(gtk.STOCK_CLOSE)
        else:
            spacing = ' ' * 3
        self.pbar.set_text('Percent Complete:%s%d' % 
                            (spacing, int(fraction * 100)) + '%')
        
    # Clean up allocated memory and remove the timer
    def destroy_progress(self, widget, data=None):
        self.timer_gen.stop()
        gtk.main_quit()
    
    def __init__(self, timer_input, title):
        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_resizable(True)

        self.window.connect("destroy", self.destroy_progress)
        self.window.set_title(title)
        self.window.set_border_width(0)
        icon = self.window.render_icon(gtk.STOCK_DIALOG_INFO, 
                                       gtk.ICON_SIZE_BUTTON)
        self.window.set_icon(icon)

        vbox = gtk.VBox(False, 1)
        vbox.set_border_width(10)
        self.window.add(vbox)
        vbox.show()
  
        # Create a centering alignment object
        align = gtk.Alignment(0.5, 0.5, 0, 0)
        vbox.pack_start(align, False, False, 1)
        align.show()

        # Create the ProgressBar
        self.pbar = gtk.ProgressBar()
        self.pbar.set_text('Percent Complete:    0%')

        align.add(self.pbar)
        self.pbar.show()

        # Add a generator callback to update the value of the progress bar
        self.timer_gen = GeneratorTask(timer_input, 
                                       self.set_progress_bar_fraction)
                
        # Add a button to exit the program
        self.button = gtk.Button(stock=gtk.STOCK_CANCEL)
        self.button.connect("clicked", self.destroy_progress)
        vbox.pack_start(self.button, False, False, 0)

        # This makes it so the button is the default.
        self.button.set_flags(gtk.CAN_DEFAULT)

        # This grabs this button to be the default button. Simply hitting
        # the "Enter" key will cause this button to activate.
        self.button.grab_default()
        self.button.show()

        self.timer_gen.start()
        self.window.show()

def fake_filter():
    import time
    for i in xrange(1,100):
        time.sleep(0.25)
        yield i/100.0
    yield 1.0

def main():
    gtk.main()
    return 0

if __name__ == "__main__":
    ProgressBar(fake_filter, 'Example Prog')
    main()

# vim:syntax=python
# vim:sw=4:softtabstop=4:expandtab
